/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALDFK31AU03DRIVER_H
#define REALDFK31AU03DRIVER_H

#include "Dfk31Au03State.h"
#include "Dfk31Au03Def.h"

#include "Utilities/Counter.h"

namespace dfk41au03 {

/*!
 * \class RealDfk31Au03Driver
 * \brief Real implementation class for the Dfk31Au03 device driver
 *
 * This class is a real implementation of Dfk31Au03 device driver.
 */
class RealDfk31Au03Driver: public Dfk31Au03State {

public:
	static char* DEV_NAME;

	/*!
	 * \brief Constructor
	 * \param[in] owner The device driver main class link
	 */
	RealDfk31Au03Driver(Dfk31Au03Driver* owner);

	/*!
	 * \brief Destructor
	 */
	virtual ~RealDfk31Au03Driver();

	/*!
	 * \brief Init command implementation
	 */
	virtual void init();
	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear();

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Read command implementation
	 */
	virtual void read();

	/*!
	 * \brief Synchronize Read command implementation
	 */
	virtual void synchroniseRead();

	/*!
	 * \brief Start command implementation
	 */
	virtual void start();

	/*!
	 * \brief Resume command implementation
	 */
	virtual void resume();

	/*!
	 * \brief Pause command implementation
	 */
	virtual void pause();

	/*!
	 * \brief Stop command implementation
	 */
	virtual void stop();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

private:
//	uint8_t *buffer;
#define CLEAR(x) memset(&(x), 0, sizeof(x))
	uint8_t *buffer;
	struct buffer {
		void *start;
		size_t length;
	};
	int16 m_Handle;
	Counter m_TimeCounter;
	unsigned char dest_buf[2359296];

	void v4lconvert_yuyv_to_rgb24(const unsigned char *src, unsigned char *dest, int width, int height, int stride);

	void errno_exit(const char *s);

	int xioctl(int fd, int request, void *arg);

	void process_image(const void *p_buf, const int len_buf);

	int read_frame(int count);

	void mainloop();

	void stop_capturing();

	void start_capturing(void);

	void uninit_device();

	void init_mmap();

	void init_device();

	void close_device();

	void open_device();

#define OUTFILE_NAME	"capture.ppm"
#define COUNT_IGNORE	2	// frame count for initilize camera

#define IMAGE_WIDTH	1024
#define IMAGE_HEIGHT	768

	int fd;
	struct buffer * buffers;
	unsigned int n_buffers;

#define CLIP(color) (unsigned char)(((color)>0xFF)?0xff:(((color)<0)?0:(color)))
	typedef unsigned char uint8x;
	typedef char int8x;

};

}
#endif //REALDFK31AU03DRIVER_H
