/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef USBPVCAMBRIDGE_H
#define USBPVCAMBRIDGE_H

/**
 * Copied definitions from master.h and pvcam.h
 */
namespace pvcam {

enum
{
  READOUT_NOT_ACTIVE = 0,
  EXPOSURE_IN_PROGRESS,
  READOUT_IN_PROGRESS,
  READOUT_COMPLETE,
  FRAME_AVAILABLE = READOUT_COMPLETE,
  READOUT_FAILED,
  ACQUISITION_IN_PROGRESS,
  MAX_CAMERA_STATUS
};

enum
{ ATTR_CURRENT, ATTR_COUNT, ATTR_TYPE, ATTR_MIN, ATTR_MAX, ATTR_DEFAULT,
  ATTR_INCREMENT, ATTR_ACCESS, ATTR_AVAIL
};

enum
{
  TIMED_MODE,
  STROBED_MODE,
  BULB_MODE,
  TRIGGER_FIRST_MODE,
  FLASH_MODE,
  VARIABLE_TIMED_MODE,
  INT_STROBE_MODE
};

enum
{ CCS_NO_CHANGE, CCS_HALT, CCS_HALT_CLOSE_SHTR, CCS_CLEAR,
  CCS_CLEAR_CLOSE_SHTR, CCS_OPEN_SHTR, CCS_CLEAR_OPEN_SHTR
};

}

#define CAM_NAME_LEN 	32
#define PARAM_SER_SIZE	((2<<16) + (6<<24)     +  58)
#define PARAM_PAR_SIZE	((2<<16) + (6<<24)     +  57)
#define ERROR_MSG_LEN  255

int plExpUninitSeq();
int plCamClose(short hCam);
int plPVCamUninit();
int plPVCamInit();
int plCamGetName(short camNum, char * camName);
int plCamOpen(char * camName, short * hCam);
int plExpFinishSeq(short hCam, void * pixelStream, short hBuf);
void * allocRegion();
void setS1(void * region, unsigned short value);
void setS2(void * region, unsigned short value);
void setP1(void * region, unsigned short value);
void setP2(void * region, unsigned short value);
void setSBin(void * region, unsigned short value);
void setPBin(void * region, unsigned short value);
int plGetParam(short hCam, unsigned long paramId, short paramAttribute, void * paramValue);
int plExpInitSeq();
int plExpSetupSeq(short hCam, unsigned short expTotal, unsigned short rgnTotal, void const * rgn, short expMode, unsigned long exposureTime, unsigned long * expBytes);
int plExpStartSeq(short hCam, void * pixelStream);
int plExpAbort(short hCam, short camState);
int plExpCheckStatus(short hCam, short * status, unsigned long * bytesArrived);
short plErrorCode();
int plErrorMessage(short errCode, char * message);



#endif //USBPVCAMBRIDGE_H
